open Core.Std
open OUnit
open Schema
open Indexes
open Block
open Purity

let parsed s =
  let lexbuf = Lexing.from_string s in
  let (sch,_) = Parser.main Lexer.token lexbuf in
  sch

open Schema_test.Values

let sch   = parsed "/\\i=1..n P_i"
let pur   = purity (Block.of_schs_and_cstr ~schs:[sch] ~cstr:Constraint.top)
let sch2  = parsed "P_1 \\/ ~P_2"
let pur2  = purity (Block.of_schs_and_cstr ~schs:[sch2] ~cstr:Constraint.top)
let sch3  = parsed "P_1 \\/ (/\\i=1..n P_i)"
let pur3  = purity (Block.of_schs_and_cstr ~schs:[sch3] ~cstr:Constraint.top)
let sch4  = parsed "P_1 /\\ /\\i=1..n (~P_i \\/ P_i+1)"
let pur4  = purity (Block.of_schs_and_cstr ~schs:[sch4] ~cstr:Constraint.top)
let sch5  = parsed "(\\/i=1..n ~P_i) /\\ /\\i=1..n (~Q_i \\/ P_i+1) /\\ P_1"
let pur5  = purity (Block.of_schs_and_cstr ~schs:[sch5] ~cstr:Constraint.top)
let cstr0 = Constraint.constraint_atom (Indexes.of_var (Variable.of_string "n")) Constraint.Gt int0
let sch6  = parsed "~P_n /\\ (/\\i=1..n-2 (~P_i \\/ P_i+1)) /\\ ~P_n-1 /\\ ~P_n /\\ P_1"
let pur6  = purity (Block.of_schs_and_cstr ~schs:[sch6] ~cstr:Constraint.top)
let cstr1 = Constraint.constraint_atom (Indexes.of_var (Variable.of_string "n")) Constraint.Gt int1

let purify ~cstr ~pur sch = 
  match Schema.purify ~cstr ~pur sch with
  |Modified.Modified,(`Top:Schema.t) -> None
  |_                                 -> Some ()

let test =
  "purity" >:::
    [ "is_pure" >::
      (fun () ->
        "~P_0 & sch"          @? (purify ~cstr:Constraint.top np0  ~pur = None);
        "P_0 & sch"           @? (purify ~cstr:Constraint.top p0   ~pur = None);
        "P_1 & sch"           @? (purify ~cstr:Constraint.top p1   ~pur = None);
        "~P_1 & sch"          @? (purify ~cstr:Constraint.top np1  ~pur <> None);
        "P_n & sch"           @? (purify ~cstr:Constraint.top p_n  ~pur = None);
        "~P_n & sch"          @? (purify ~cstr:Constraint.top np_n ~pur <> None);
        "P_n+1 & sch"         @? (purify ~cstr:Constraint.top p_nplus1   ~pur = None);
        "~P_n+1 & sch"        @? (purify ~cstr:Constraint.top np_nplus1  ~pur = None);
        "P_n-1 & sch"         @? (purify ~cstr:Constraint.top p_nminus1  ~pur = None);
        "~P_n-1 & sch"        @? (purify ~cstr:Constraint.top np_nminus1 ~pur <> None);
        "P_1 & sch2"          @? (purify ~cstr:Constraint.top p1  ~pur:pur2 = None);
        "~P_1 & sch2"         @? (purify ~cstr:Constraint.top np1 ~pur:pur2 <> None);
        "P_2 & sch2"          @? (purify ~cstr:Constraint.top p2  ~pur:pur2 <> None);
        "~P_2 & sch2"         @? (purify ~cstr:Constraint.top np2 ~pur:pur2 = None);
        "~P_0 & sch3"         @? (purify ~cstr:Constraint.top np0 ~pur:pur3 = None);
        "P_0 & sch3"          @? (purify ~cstr:Constraint.top p0  ~pur:pur3 = None);
        "P_1 & sch3"          @? (purify ~cstr:Constraint.top p1  ~pur:pur3 = None);
        "~P_1 & sch3"         @? (purify ~cstr:Constraint.top np1 ~pur:pur3 <> None);
        "P_n & sch3"          @? (purify ~cstr:Constraint.top p_n ~pur:pur3 = None);
        "~P_n & sch3"         @? (purify ~cstr:Constraint.top np_n ~pur:pur3 <> None);
        "P_n+1 & sch3"        @? (purify ~cstr:Constraint.top p_nplus1  ~pur:pur3 = None);
        "~P_n+1 & sch3"       @? (purify ~cstr:Constraint.top np_nplus1 ~pur:pur3 <> None);
        "~P_n+1 & sch3 | n>0" @? (purify ~cstr:cstr0 np_nplus1 ~pur:pur3 = None);
        "P_n-1 & sch3"        @? (purify ~cstr:Constraint.top p_nminus1  ~pur:pur3 = None);
        "~P_n-1 & sch3"       @? (purify ~cstr:Constraint.top np_nminus1 ~pur:pur3 <> None);
        "P_1 & sch3"          @? (purify ~cstr:Constraint.top p1  ~pur:pur3 = None);
        "~P_1 & sch3"         @? (purify ~cstr:Constraint.top np1 ~pur:pur3 <> None);
        "P_1 & sch4"          @? (purify ~cstr:Constraint.top p1  ~pur:pur4 <> None);
        "~P_1 & sch4"         @? (purify ~cstr:Constraint.top np1 ~pur:pur4 <> None);
        "P_2 & sch4"          @? (purify ~cstr:Constraint.top p2  ~pur:pur4 <> None);
        "~P_2 & sch4"         @? (purify ~cstr:Constraint.top np2 ~pur:pur4 <> None);
        "P_n-1 & sch4"        @? (purify ~cstr:Constraint.top p_nminus1  ~pur:pur4 <> None);
        "~P_n-1 & sch4"       @? (purify ~cstr:Constraint.top np_nminus1 ~pur:pur4 <> None);
        "P_n & sch4"          @? (purify ~cstr:Constraint.top p_n  ~pur:pur4 <> None);
        "~P_n & sch4"         @? (purify ~cstr:Constraint.top np_n ~pur:pur4 <> None);
        "P_n+1 & sch4"        @? (purify ~cstr:Constraint.top p_nplus1  ~pur:pur4 = None);
        "~P_n+1 & sch4"       @? (purify ~cstr:Constraint.top np_nplus1 ~pur:pur4 <> None);
        "P_1 & sch5"          @? (purify ~cstr:Constraint.top p1  ~pur:pur5 <> None);
        "~P_1 & sch5"         @? (purify ~cstr:Constraint.top np1 ~pur:pur5 <> None);
        "P_2 & sch5"          @? (purify ~cstr:Constraint.top p1  ~pur:pur5 <> None);
        "~P_2 & sch5"         @? (purify ~cstr:Constraint.top np2 ~pur:pur5 <> None);
        "P_n-1 & sch5"        @? (purify ~cstr:Constraint.top p_nminus1  ~pur:pur5 <> None);
        "~P_n-1 & sch5"       @? (purify ~cstr:Constraint.top np_nminus1 ~pur:pur5 <> None);
        "P_n & sch5"          @? (purify ~cstr:Constraint.top p_n  ~pur:pur5 <> None);
        "~P_n & sch5"         @? (purify ~cstr:Constraint.top np_n ~pur:pur5 <> None);
        "P_n+1 & sch5"        @? (purify ~cstr:Constraint.top p_nplus1  ~pur:pur5 = None);
        "~P_n+1 & sch5"       @? (purify ~cstr:Constraint.top np_nplus1 ~pur:pur5 <> None);
        "~P_n & sch6 | n>1"   @? (purify ~cstr:cstr1 np_n ~pur:pur6 = None);
      );
      (*
       * (let prop_and_pure_strat sch =
       *   Regstab.prop_rules ~k:(Regstab.pure_rule ~k:Strategy.ident_strat) ~applied:false ~schs:[sch] ~cstr:Constraint.top in
       * Propositional_test.parametrized_test prop_and_pure_strat 
       * );
       *)
    ]
